/*
(c) Janusz Ganczarski
http://www.januszg.hg.pl
JanuszG@enter.net.pl
*/

void main ()
{
    // przeksztacenie wektora normalnego gl_Normal do wsprzdnych ukadu obserwatora
    // przy uyciu macierzy gl_NormalMatrix; jest to standardowe przeksztacenie
    // klasycznego potoku OpenGL; otrzymany rezultat jest normalizowany
    vec3 N = normalize (gl_NormalMatrix * gl_Normal);

    // przeksztacenie biecego wierzchoka do wsprzdnych ukadu obserwatora
    vec3 V = vec3 (gl_ModelViewMatrix * gl_Vertex);

    // obliczenie odlegoci biecego wierzchoka od pooenia rda wiata
    float distance = length (gl_LightSource[0].position.xyz - V);

    // normalizacja wektora kierunku wiata do biecego wierzchoka
    vec3 L = normalize (gl_LightSource[0].position.xyz - V);

    // obliczenie wspczynnika rozproszenia wiata
    float att = 1.0 / (gl_LightSource[0].constantAttenuation +
                gl_LightSource[0].linearAttenuation * distance +
                gl_LightSource[0].quadraticAttenuation * distance * distance);

    // obliczenie cosinusa kta pomidzy wektorem normalnym a wektorem
    // kierunku wiata; poniewa oba wektory s jednostkowe ich iloczyn
    // skalarny jest rwny poszukiwanemu cosinusowi
    float NdotL = max (dot (N,L),0.0);
    
    // warto pocztkowa skadowych wiata odbitego
    vec4 specular = vec4 (0.0);

    // obliczenia zwizane ze wiatem odbitym wykonywane s tylko dla owietlonych powierzchni
    if (NdotL > 0.0)
    {
        // normalizacja wektora powkowego i obliczenie cosinusa
        // kta pomidzy wektorem normalnym a wektorem powkowym
        float NdotH = max (dot (N,normalize (gl_LightSource[0].halfVector.xyz)),0.0);

        // obliczenie wartoci skadowych wiata odbitego
        specular = gl_FrontMaterial.specular * gl_LightSource[0].specular * pow (NdotH,gl_FrontMaterial.shininess);
    }

    // obliczenie wartoci skadowych koloru wierzchoka
    gl_FragColor = att * (NdotL * gl_FrontMaterial.diffuse * gl_LightSource[0].diffuse +
                   gl_LightSource [0].ambient * gl_FrontMaterial.ambient + specular) +
                   gl_LightModel.ambient * gl_FrontMaterial.ambient;

    // standardowe przeksztacenie wierzchokw
    gl_Position = ftransform ();
}
